@@ -1,3 +1,27 @@
+/***********************************************************************
+ *  avra - Assembler for the Atmel AVR microcontroller series
+ *  Copyright (C) 1998-1999 Jon Anders Haugum
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; see the file COPYING.  If not, write to
+ *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ *  Boston, MA 02111-1307, USA.
+ *
+ *
+ *  Author of avra can be reached at:
+ *     email: jonah@omegav.ntnu.no
+ *     www: http://www.omegav.ntnu.no/~jonah/el/avra.html
+ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -199,8 +223,8 @@ int parse_directive(struct prog_info *pi, int pass)
 					{
 					if(pass == PASS_2)
 						{
-						write_ee_byte(pi, pi->eseg_addr, i & 0xff);
-						write_ee_byte(pi, pi->eseg_addr + 1, (i >> 8) & 0xff);
+						write_ee_byte(pi, pi->eseg_addr, (unsigned char)i);
+						write_ee_byte(pi, pi->eseg_addr + 1, (unsigned char)(i >> 8));
 						}
 					pi->eseg_addr += 2;
 					if(pass == PASS_1)
@@ -443,7 +467,7 @@ int parse_directive(struct prog_info *pi, int pass)
 				return(False);
 				}
 			strcpy(label->name, next);
-			label->value = 1;
+			label->value = i;
 			break;
 		case DIRECTIVE_UNDEF: // TODO
 			break;
@@ -480,7 +504,7 @@ int parse_directive(struct prog_info *pi, int pass)
 		case DIRECTIVE_IF:
 			if(!next)
 				{
-				print_msg(pi, MSGTYPE_ERROR, ".IFDEF needs an operand");
+				print_msg(pi, MSGTYPE_ERROR, ".IF needs an expression");
 				return(True);
 				}
 			get_next_token(next, TERM_END);
@@ -561,8 +585,8 @@ char *term_string(struct prog_info *pi, char *string)
 		print_msg(pi, MSGTYPE_ERROR, "String must be enclosed in \"-signs");
 	else
 		string++;
-	for(i = 0; (string[i] != '\"') && !IS_END(string[i]); i++);
-	if(IS_END(string[i]))
+	for(i = 0; (string[i] != '\"') && !((string[i] == 10) || (string[i] == 13) || (string[i] == '\0')); i++);
+	if((string[i] == 10) || (string[i] == 13) || (string[i] == '\0'))
 		print_msg(pi, MSGTYPE_ERROR, "String is missing a closing \"-sign");
 	string[i] = '\0';
 	return(string);
@@ -600,7 +624,7 @@ int parse_db(struct prog_info *pi, char *next, int pass)
 			if((i < -128) || (i > 255))
 				print_msg(pi, MSGTYPE_ERROR, "Value %d is out of range (-128 <= k <= 255)", i);
 			count++;
-			write_db(pi, i, &prev, count, pass);
+			write_db(pi, (char)i, &prev, count, pass);
 			}
 		next = data;
 		}
@@ -634,7 +658,7 @@ void write_db(struct prog_info *pi, char byte, char *prev, int count, int pass)
 		if((count % 2) == 0)
 			{
 			if(pass == PASS_2)
-				write_prog_word(pi, pi->cseg_addr, (byte << 8) | *prev);
+				write_prog_word(pi, pi->cseg_addr, (byte << 8) | (*prev & 0xff));
 			pi->cseg_addr++;
 			if(pass == PASS_1)
 				pi->cseg_count++;
@@ -647,70 +671,98 @@ void write_db(struct prog_info *pi, char byte, char *prev, int count, int pass)
 
 int spool_conditional(struct prog_info *pi, int only_endif)
 	{
-	int i, current_depth = 0;
-	char *next;
+	int current_depth = 0, do_next;
 
-	for(;;)
+	if(pi->macro_line)
 		{
-		if(fgets(pi->fi->buff, LINEBUFFER_LENGTH, pi->fi->fp))
+		while((pi->macro_line = pi->macro_line->next))
+			{
+			pi->macro_call->line_index++;
+			if(check_conditional(pi, pi->macro_line->line, &current_depth,  &do_next, only_endif))
+				{
+				if(!do_next)
+					return(True);
+				}
+			else
+				return(False);
+			}
+		print_msg(pi, MSGTYPE_ERROR, "Found no closing .ENDIF in macro");
+		}
+	else
+		{
+		while(fgets(pi->fi->buff, LINEBUFFER_LENGTH, pi->fi->fp))
 			{
 			pi->fi->line_number++;
-			i = 0;
-			while(IS_HOR_SPACE(pi->fi->buff[i]) && !IS_END(pi->fi->buff[i])) i++;
-			if(pi->fi->buff[i] == '.')
+			if(check_conditional(pi, pi->fi->buff, &current_depth,  &do_next, only_endif))
 				{
-				i++;
-				if(!nocase_strncmp(&pi->fi->buff[i], "if", 2))
-				        current_depth++;
-				else if(!nocase_strncmp(&pi->fi->buff[i], "endif", 5))
-				        {
-					if(current_depth == 0)
-					        return(True);
-					current_depth--;
-					}
-				else if(!only_endif && (current_depth == 0))
-				        {
-					if(!nocase_strncmp(&pi->fi->buff[i], "else", 4))
-					        {
-						pi->conditional_depth++;
-					        return(True);
-						}
-					else if(!nocase_strncmp(&pi->fi->buff[i], "elif", 4))
-					        {
-						next = get_next_token(&pi->fi->buff[i], TERM_SPACE);
-						if(!next)
-						        {
-							print_msg(pi, MSGTYPE_ERROR, ".ELIF needs an operand");
-							return(True);
-							}
-						get_next_token(next, TERM_END);
-						if(!get_expr(pi, next, &i))
-						        return(False);
-						if(i)
-						        pi->conditional_depth++;
-						else
-						        {
-						        if(!spool_conditional(pi, False))
-							        return(False);
-							}
-						return(True);
-						}
-					}
+				if(!do_next)
+					return(True);
 				}
+			else
+				return(False);
+			}
+		if(feof(pi->fi->fp))
+			{
+			print_msg(pi, MSGTYPE_ERROR, "Found no closing .ENDIF");
+			return(True);
 			}
 		else
 			{
-			if(feof(pi->fi->fp))
-				{
-				print_msg(pi, MSGTYPE_ERROR, "Found no closing .ENDIF");
-				return(True);
-				}
-			else
-				{
-				perror(pi->fi->include_file->name);
-				return(False);
-				}
+			perror(pi->fi->include_file->name);
+			return(False);
 			}
 		}
 	return(True);
 	}
+
+
+int check_conditional(struct prog_info *pi, char *buff, int *current_depth, int *do_next, int only_endif)
+	{
+	int i = 0;
+	char *next;
+
+	*do_next = False;
+	while(IS_HOR_SPACE(buff[i]) && !IS_END(buff[i])) i++;
+	if(buff[i] == '.')
+		{
+		i++;
+		if(!nocase_strncmp(&buff[i], "if", 2))
+			(*current_depth)++;
+		else if(!nocase_strncmp(&buff[i], "endif", 5))
+		        {
+			if(*current_depth == 0)
+			        return(True);
+			(*current_depth)--;
+			}
+		else if(!only_endif && (*current_depth == 0))
+		        {
+			if(!nocase_strncmp(&buff[i], "else", 4))
+			        {
+				pi->conditional_depth++;
+			        return(True);
+				}
+			else if(!nocase_strncmp(&buff[i], "elif", 4))
+			        {
+				next = get_next_token(&buff[i], TERM_SPACE);
+				if(!next)
+				        {
+					print_msg(pi, MSGTYPE_ERROR, ".ELIF needs an operand");
+					return(True);
+					}
+				get_next_token(next, TERM_END);
+				if(!get_expr(pi, next, &i))
+				        return(False);
+				if(i)
+				        pi->conditional_depth++;
+				else
+				        {
+				        if(!spool_conditional(pi, False))
+					        return(False);
+					}
+				return(True);
+				}
+			}
+		}
+	*do_next = True;
+	return(True);
+	}
