@@ -1,3 +1,27 @@
+/***********************************************************************
+ *  avra - Assembler for the Atmel AVR microcontroller series
+ *  Copyright (C) 1998-1999 Jon Anders Haugum
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; see the file COPYING.  If not, write to
+ *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ *  Boston, MA 02111-1307, USA.
+ *
+ *
+ *  Author of avra can be reached at:
+ *     email: jonah@omegav.ntnu.no
+ *     www: http://www.omegav.ntnu.no/~jonah/el/avra.html
+ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -581,7 +605,8 @@ int get_mnemonic_type(char *mnemonic)
 	int i;
 
 	for(i = 0; i < MNEMONIC_COUNT; i++)
-		if(!strcmp(mnemonic, instruction_list[i].mnemonic)) return(i);
+		if(!strcmp(mnemonic, instruction_list[i].mnemonic))
+			return(i);
 	return(-1);
 	}
 
@@ -591,6 +616,12 @@ int get_register(struct prog_info *pi, char *data)
 	int reg = 0;
 	struct def *def;
 
+	for(def = pi->first_def; def; def = def->next)
+		if(!nocase_strcmp(def->name, data))
+			{
+			reg = def->reg;
+			return(reg);
+			}
 	if((tolower(data[0]) == 'r') && isdigit(data[1]))
 		{
 		reg = atoi(&data[1]);
@@ -598,15 +629,7 @@ int get_register(struct prog_info *pi, char *data)
 			print_msg(pi, MSGTYPE_ERROR, "R%d is not a valid register", reg);
 		}
 	else
-		{
-		for(def = pi->first_def; def; def = def->next)
-			if(!nocase_strcmp(def->name, data))
-				{
-				reg = def->reg;
-				return(reg);
-				}
 		print_msg(pi, MSGTYPE_ERROR, "No register associated with %s", data);
-		}
 	return(reg);
 	}
 
@@ -626,10 +649,15 @@ int get_bitnum(struct prog_info *pi, char *data, int *ret)
 
 int get_indirect(struct prog_info *pi, char *operand)
 	{
+	int i = 1;
+
 	switch(tolower(operand[0]))
 		{
 		case '-':
-			switch(tolower(operand[1]))
+			while(IS_HOR_SPACE(operand[i])) i++;
+			if(operand[i + 1] != '\0')
+				print_msg(pi, MSGTYPE_ERROR, "Garbage in operand (%s)", operand);
+			switch(tolower(operand[i]))
 				{
 				case 'x':
 					return(2);
@@ -642,25 +670,40 @@ int get_indirect(struct prog_info *pi, char *operand)
 					return(0);
 				}
 		case 'x':
-			if(operand[1] == '+')
+			while(IS_HOR_SPACE(operand[i])) i++;
+			if(operand[i] == '+')
+				{
+				if(operand[i + 1] != '\0')
+					print_msg(pi, MSGTYPE_ERROR, "Garbage in operand (%s)", operand);
 				return(1);
-			else if(operand[1] == '\0')
+				}
+			else if(operand[i] == '\0')
 				return(0);
 			else
 				print_msg(pi, MSGTYPE_ERROR, "Garbage after operand (%s)", operand);
 			return(0);
 		case 'y':
-			if(operand[1] == '+')
+			while(IS_HOR_SPACE(operand[i])) i++;
+			if(operand[i] == '+')
+				{
+				if(operand[i + 1] != '\0')
+					print_msg(pi, MSGTYPE_ERROR, "Garbage in operand (%s)", operand);
 				return(4);
-			else if(operand[1] == '\0')
+				}
+			else if(operand[i] == '\0')
 				return(3);
 			else
 				print_msg(pi, MSGTYPE_ERROR, "Garbage after operand (%s)", operand);
 			return(0);
 		case 'z':
-			if(operand[1] == '+')
+			while(IS_HOR_SPACE(operand[i])) i++;
+			if(operand[i] == '+')
+				{
+				if(operand[i + 1] != '\0')
+					print_msg(pi, MSGTYPE_ERROR, "Garbage in operand (%s)", operand);
 				return(7);
-			else if(operand[1] == '\0')
+				}
+			else if(operand[i] == '\0')
 				return(6);
 			else
 				print_msg(pi, MSGTYPE_ERROR, "Garbage after operand (%s)", operand);
